@@ -9,23 +9,31 @@ package Template::Declare::Tags;
 our $VERSION = '0.40';
 
 use Template::Declare;
-use vars qw( @EXPORT_OK $PRIVATE $self @TagSubs );
 use base 'Exporter';
 use Carp qw(carp croak);
 use Symbol 'qualify_to_ref';
 
-our @EXPORT
-    = qw( with template private show show_page attr outs
-          outs_raw in_isolation $self under
-          get_current_attr xml_decl
-          smart_tag_wrapper current_template create_wrapper );
+our $self;
+
+our @EXPORT = qw(
+    template private current_template
+    show show_page
+    attr with get_current_attr 
+    outs outs_raw
+    xml_decl
+    under setting
+    smart_tag_wrapper create_wrapper
+    $self
+);
+
 our @TAG_SUB_LIST;
+our @TagSubs;
 *TagSubs = \@TAG_SUB_LIST;  # For backward compatibility only
 
 our %ATTRIBUTES       = ();
 our %ELEMENT_ID_CACHE = ();
 our $TAG_NEST_DEPTH   = 0;
-our @TEMPLATE_STACK;
+our @TEMPLATE_STACK   = ();
 
 our $SKIP_XML_ESCAPING = 0;
 
@@ -70,7 +78,7 @@ sub import {
         Template::Declare::Tags::install_tag($_, $tagset)
             for @$tag_list;
     }
-   __PACKAGE__->export_to_level(1, $self);
+    __PACKAGE__->export_to_level(1, $self);
 }
 
 sub _install {
@@ -106,15 +114,18 @@ Template::Declare::Tags - Build and install XML Tag subroutines for Template::De
         img { src is 'dog.gif' }
     };
 
-    # Produces:
-    # <link />
-    # <table>
-    #  <tr>
-    #   <td>Hello, world!</td>
-    #  </tr>
-    # </table>
-    # <img src="cat.gif" />
-    # <img src="dog.gif" />
+Produces:
+
+ <link />
+ <table>
+  <tr>
+   <td>Hello, world!</td>
+  </tr>
+ </table>
+ <img src="cat.gif" />
+ <img src="dog.gif" />
+
+Using XUL templates with a namespace:
 
     package MyApp::Templates;
 
@@ -130,26 +141,28 @@ Template::Declare::Tags - Build and install XML Tag subroutines for Template::De
         }
     };
 
-    # Produces:
-    #   <groupbox>
-    #    <caption label="Colors" />
-    #    <html:div>
-    #     <html:p>howdy!</html:p>
-    #    </html:div>
-    #    <html:br></html:br>
-    #   </groupbox>
+Produces:
+
+ <groupbox>
+  <caption label="Colors" />
+  <html:div>
+   <html:p>howdy!</html:p>
+  </html:div>
+  <html:br></html:br>
+ </groupbox>
 
 =head1 DESCRIPTION
 
-C<Template::Declare::Tags> is used to generate and install
-subroutines for tags into the user's namespace.
+C<Template::Declare::Tags> is used to generate templates and install
+subroutines for tag sets into the calling namespace.
 
-You can specify the tag sets used by providing a list of
-module list in the C<use> statement:
+You can specify the tag sets to install by providing a list of tag modules in
+the C<use> statement:
 
     use Template::Declare::Tags qw/ HTML XUL /;
 
-By default, it uses the tag set provided by L<Template::Declare::TagSet::HTML>. So
+By default, Template::Declare::Tags uses the tag set provided by
+L<Template::Declare::TagSet::HTML>. So
 
     use Template::Declare::Tags;
 
@@ -158,33 +171,33 @@ is equivalent to
     use Template::Declare::Tags 'HTML';
 
 Currently L<Template::Declare> bundles the following tag sets:
-L<Template::Declare::TagSet::HTML>, L<Template::Declare::TagSet::XUL>, L<Template::Declare::TagSet::RDF>, and L<Template::Declare::TagSet::RDF::EM>.
+L<Template::Declare::TagSet::HTML>, L<Template::Declare::TagSet::XUL>,
+L<Template::Declare::TagSet::RDF>, and L<Template::Declare::TagSet::RDF::EM>.
 
-You can certainly specify your own tag set classes, as long
-as they subclass L<Template::Declare::TagSet> and implement
-the corresponding methods (e.g. C<get_tag_list>).
+You can specify your own tag set classes, as long as they subclass
+L<Template::Declare::TagSet> and implement the corresponding methods (e.g.
+C<get_tag_list>).
 
 If you implement a custom tag set module named
-C<Template::Declare::TagSet::Foo>.
+C<Template::Declare::TagSet::Foo>, you can load it into a template module like
+so:
 
- use Template::Declare::Tags 'Foo';
+    use Template::Declare::Tags 'Foo';
 
-If you give the your tag set module a different name, say, C<MyTag::Foo>, then
-you use the C<from> option:
+If your tag set module is not under the
+L<Template::Declare::TagSet|Template::Declare::TagSet> namespace, use the
+C<from> option to load it. Fore example, if you created a tag set named
+C<MyTag::Foo>, then you could load it like so:
 
- use Template::Declare::Tags Foo => { from => 'MyTag::Foo' };
+    use Template::Declare::Tags Foo => { from => 'MyTag::Foo' };
 
-Then C<Template::Declare::Tags> will no longer try to load C<Template::Declare::TagSet::Foo>
-and C<MyTag::Foo> will be loaded instead.
-
-XML namespaces are emulated by Perl packages. For
-example, you can embed HTML tags within XUL using the C<html> namespace:
+XML namespaces are emulated by Perl packages. For example, to embed HTML tags
+within XUL using the C<html> namespace:
 
     package MyApp::Templates;
 
     use base 'Template::Declare';
-    use Template::Declare::Tags
-        'XUL', HTML => { namespace => 'html' };
+    use Template::Declare::Tags 'XUL', HTML => { namespace => 'html' };
 
     template main => sub {
         groupbox {
@@ -194,27 +207,33 @@ example, you can embed HTML tags within XUL using the C<html> namespace:
         }
     };
 
-This will give you
+This will output:
 
-       <groupbox>
-        <caption label="Colors" />
-        <html:div>
-         <html:p>howdy!</html:p>
-        </html:div>
-        <html:br></html:br>
-       </groupbox>
+ <groupbox>
+  <caption label="Colors" />
+  <html:div>
+   <html:p>howdy!</html:p>
+  </html:div>
+  <html:br></html:br>
+ </groupbox>
 
-Behind the scene, C<Template::Declare::Tags>  will generate a Perl package named C<html> and install HTML tag subroutines into that package. On the other hand, XUL tag subroutines are installed into the current package, namely, C<MyApp::Templates> in the previous example.
+Behind the scenes, C<Template::Declare::Tags> generates a Perl package named
+C<html> and installs the HTML tag subroutines into that package. On the other
+hand, XUL tag subroutines are installed into the current package, namely,
+C<MyApp::Templates> in the previous example.
 
-There are cases when you want to specify a different Perl package for a perticular XML namespace name. For instance, the C<html> Perl package has already been used for other purposes in your application and you don't want to install subs there and mess things up, then the C<package> option can come to rescue:
+There may be cases when you want to specify a different Perl package for a
+particular XML namespace. For instance, if the C<html> Perl package has
+already been used for other purposes in your application and you don't want to
+install subs there and mess things up, use the C<package> option to install
+them elsewhere:
 
     package MyApp::Templates;
     use base 'Template::Declare';
-    use Template::Declare::Tags
-        'XUL', HTML => {
-            namespace => 'htm',
-            package => 'MyHtml'
-        };
+    use Template::Declare::Tags 'XUL', HTML => {
+        namespace => 'htm',
+        package   => 'MyHtml'
+    };
 
     template main => sub {
         groupbox {
@@ -224,27 +243,42 @@ There are cases when you want to specify a different Perl package for a perticul
         }
     };
 
-This code snippet will still generate something like the following:
+This code will generate something like the following:
 
-    <groupbox>
-     <caption label="Colors" />
-     <htm:div>
-      <htm:p>howdy!</htm:p>
-     </htm:div>
-     <htm:br></htm:br>
-    </groupbox>
+ <groupbox>
+  <caption label="Colors" />
+  <htm:div>
+   <htm:p>howdy!</htm:p>
+  </htm:div>
+  <htm:br></htm:br>
+ </groupbox>
 
 =head1 METHODS AND SUBROUTINES
 
-=head2 template TEMPLATENAME => sub { 'Implementation' };
+=head2 Declaring templates
 
-C<template> declares a template in the current package. You can pass
-any url-legal characters in the template name. C<Template::Declare>
-will encode the template as a perl subroutine and stash it to be called
-with C<show()>.
+=head3 template TEMPLATENAME => sub { 'Implementation' };
+
+    template select_list => sub {
+        my $self = shift;
+        select {
+            option { $_ } for @_;
+        }
+    };
+
+Declares a template in the current package. The first argument to the template
+subroutine will always be a C<Template::Declare> object. Subsequent arguments
+will be all those passed to C<show()>. For example, to use the above example
+to output a select list of colors, you'd call it like so:
+
+    Template::Declare->show('select_list', qw(red yellow green purple));
+
+You can use any URL-legal characters in the template name;
+C<Template::Declare> will encode the template as a Perl subroutine and stash
+it where C<show()> can find it.
 
 (Did you know that you can have characters like ":" and "/" in your Perl
-subroutine names? The easy way to get at them is with "can").
+subroutine names? The easy way to get at them is with C<can>).
 
 =cut
 
@@ -267,76 +301,27 @@ sub template ($$) {
          # We're being called by something like private that doesn't want us to register ourselves
         return ( $template_class, $template_name, $codesub );
     } else {
-
        # We've been called in a void context and should register this template
-        Template::Declare::register_template( $template_class, $template_name,
-            $codesub );
+        Template::Declare::register_template(
+            $template_class,
+            $template_name,
+            $codesub,
+        );
     }
-
 }
 
-=head2 create_wrapper WRAPPERNAME => sub { 'Implementation' };
+=head3 private template TEMPLATENAME => sub { 'Implementation' };
 
-C<create_wrapper> declares a wrapper subroutine that can be called like a tag
-sub, but can optionally take arguments to be passed to the wrapper sub. For
-example, if you wanted to wrap all of the output of a template in the usual
-HTML headers and footers, you can do something like this:
+    private template select_list => sub {
+        my $self = shift;
+        select {
+            option { $_ } for @_;
+        }
+    };
 
-  package MyApp::Templates;
-  use Template::Declare::Tags;
-  use base 'Template::Declare';
-
-  BEGIN {
-      create_wrapper wrap => sub {
-          my $code = shift;
-          my %params = @_;
-          html {
-              head { title { outs "Hello, $params{user}!"} };
-              body {
-                  $code->();
-                  div { outs 'This is the end, my friend' };
-              };
-          }
-      };
-  }
-
-  template inner => sub {
-      wrap {
-          h1 { outs "Hello, Jesse, s'up?" };
-      } user => 'Jesse';
-  };
-
-Note how the C<wrap> wrapper function is available for calling after it has
-been declared in a C<BEGIN> block. Also note how you can pass arguments to the
-function after the closing brace (you don't need a comma there!).
-
-The output from the "inner" template will look something like this:
-
-  <html>
-   <head>
-    <title>Hello, Jesse!</title>
-   </head>
-   <body>
-    <h1>Hello, Jesse, s&#39;up?</h1>
-    <div>This is the end, my friend</div>
-   </body>
-  </html>
-
-=cut
-
-sub create_wrapper ($$) {
-    my $wrapper_name   = shift;
-    my $coderef        = shift;
-    my $template_class = caller;
-
-    # Shove the code ref into the calling class.
-    no strict 'refs';
-    *{"$template_class\::$wrapper_name"} = sub (&;@) { goto $coderef };
-}
-
-=head2 private template TEMPLATENAME => sub { 'Implementation' };
-
-C<private> declares that a template isn't available to be called directly from client code.
+Declares that a template isn't available to be called directly from client
+code. The resulting template can instead only be called from the package in
+which it's created.
 
 =cut
 
@@ -347,27 +332,83 @@ sub private (@) {
     Template::Declare::register_private_template( $class, $subname, $code );
 }
 
-=head2 attr HASH
+=head2 Showing templates
 
-With C<attr>, you can specify attributes for HTML tags.
+=head3 show [$template_name or $template_coderef], args
 
+    show( main => { user => 'Bob' } );
 
-Example:
+Displays templates. The first argument is the name of the template to be
+displayed. Any additional arguments will be passed directly to the template.
 
- p {
-    attr { class => 'greeting text',
-           id    => 'welcome' };
-    'This is a welcoming paragraph';
- }
+C<show> can either be called with a template name or a package/object and a
+template. (It's both functional and OO.)
 
-Tag attributes can also be specified by using C<is>, as in
+If called from within a Template::Declare subclass, then private templates are
+accessible and visible. If called from something that isn't a
+Template::Declare, only public templates will be visible.
 
- p {
-    class is 'greeting text';
-    id    is 'welcome';
-    'This is a welcoming paragraph';
- }
+From the outside world, users can either call C<< Template::Declare->show() >>,
+C<< show() >> exported from Template::Declare::Tags or
+C<Template::Declare::Tags::show()> directly to render a publicly visible template.
 
+Private templates may only be called from within the C<Template::Declare>
+package.
+
+=cut
+
+sub show {
+    my $template = shift;
+
+    # if we're inside a template, we should show private templates
+    if ( caller->isa('Template::Declare') ) {
+        _show_template( $template, 1, \@_ );
+        return Template::Declare->buffer->data;
+    } else {
+        show_page( $template, @_);
+    }
+
+}
+
+=head3 show_page
+
+    show_page( main => { user => 'Bob' } );
+
+Like C<show()>, but does not dispatch to private templates. It's used
+internally by C<show()> when when that method is called from outside a
+template class.
+
+=cut
+
+sub show_page {
+    my $template = shift;
+    my $args = \@_;
+
+    Template::Declare->buffer->push(
+        private => defined wantarray,
+        from => "T::D path $template",
+    );
+    _show_template( $template, 0, $args );
+    %ELEMENT_ID_CACHE = ();
+    return Template::Declare->buffer->pop;
+}
+
+=head2 Attributes
+
+=head3 attr HASH
+
+    attr { src => 'logo.png' };
+
+Specifies attributes for the element tag in which it appears. For example, to
+add a class and ID to an HTML paragraph:
+
+    p {
+       attr {
+           class => 'greeting text',
+           id    => 'welcome',
+       };
+       'This is a welcoming paragraph';
+    }
 
 =cut
 
@@ -382,84 +423,89 @@ sub attr (&;@) {
     return @_;
 }
 
-sub append_attr {
-    die "Subroutine attr failed: $_[0] => '$_[1]'\n\t".
-        "(Perhaps you're using an unknown tag in the outer container?)";
-}
+=head3 ATTR is VALUE
 
-=head2 xml_decl HASH
+Attributes can also be specified by using C<is>, as in
 
-Emits XML declarators.
+    p {
+       class is 'greeting text';
+       id    is 'welcome';
+       'This is a welcoming paragraph';
+    }
 
-For example,
+A few tricks work for 'is':
 
-    xml_decl { 'xml', version => '1.0' };
-    xml_decl { 'xml-stylesheet',  href => "chrome://global/skin/", type => "text/css" };
+    http_equiv is 'foo'; # => http-equiv="foo"
+    xml__lang is 'foo';  # => xml:lang="foo"
 
-will produce
-
-    <?xml version="1.0"?>
-    <?xml-stylesheet href="chrome://global/skin/" type="text/css"?>
+So double underscore replaced with colon and single underscore with dash.
 
 =cut
 
-sub xml_decl (&;$) {
-    my $code = shift;
-    my @rv   = $code->();
-    my $name = shift @rv;
-    outs_raw("<?$name");
-    while ( my ( $field, $val ) = splice( @rv, 0, 2 ) ) {
-        # only defined whle in a tag context
-        outs_raw(qq/ $field="$val"/);
+# 'is' is declared later, when needed, using 'local *is::AUTOLOAD = sub {};'
+
+=head3 with
+
+    with ( id => 'greeting', class => 'foo' ),
+        p { 'Hello, World wide web' };
+
+An alternative way to specify attributes for a tag, just for variation. The
+standard way to do the same as this example using C<attr> is:
+
+    p { attr { id => 'greeting', class => 'foo' }
+        'Hello, World wide web' };
+
+=cut
+
+sub with (@) {
+    %ATTRIBUTES = ();
+    while ( my ( $key, $val ) = splice( @_, 0, 2 ) ) {
+        no warnings 'uninitialized';
+        $ATTRIBUTES{$key} = $val;
+
+        if ( lc($key) eq 'id' ) {
+            if ( $ELEMENT_ID_CACHE{$val}++ ) {
+                warn
+                    "HTML appears to contain illegal duplicate element id: $val";
+            }
+        }
+
     }
-    outs_raw("?>\n");
-    return @_;
+    wantarray ? () : '';
 }
 
-=head2 outs STUFF
+=head2 Displaying text and raw data
 
-C<outs> HTML-encodes its arguments and appends them to C<Template::Declare>'s output buffer.
+=head3 outs STUFF
 
-=head2 outs_raw STUFF
+    p { outs 'Grettings & welcome pyoonie hyoomon.' }
 
-C<outs_raw> appends its arguments to C<Template::Declare>'s output buffer without doing any HTML escaping.
+HTML-encodes its arguments and appends them to C<Template::Declare>'s output
+buffer. This is similar to simply returning a string from a tag function call,
+but is occasionally useful when you need to output a mix of things, as in:
+
+    p { outs 'hello'; em { 'world' } }
+
+=head3 outs_raw STUFF
+
+   p { outs_raw "That's what <em>I'm</em> talking about!' }
+
+Appends its arguments to C<Template::Declare>'s output buffer without HTML
+escaping.
 
 =cut
 
 sub outs     { _outs( 0, @_ ); }
 sub outs_raw { _outs( 1, @_ ); }
 
-sub _outs {
-    my $raw     = shift;
-    my @phrases = (@_);
+=head2 Installing tags and wrapping stuff
 
-    Template::Declare->buffer->push( private => (defined wantarray and not wantarray), from => "T::D outs" );
+=head3 install_tag TAGNAME, TAGSET
 
-    foreach my $item ( grep {defined} @phrases ) {
-        my $returned = ref($item) eq 'CODE'
-            ? $item->()
-            : $raw
-                ? $item
-                : _postprocess($item);
-        Template::Declare->buffer->append( $returned );
-    }
-    return Template::Declare->buffer->pop;
-}
+    install_tag video => 'Template::Declare::TagSet::HTML';
 
-=head2 get_current_attr
-
-Help! I'm deprecated/
-
-=cut
-
-sub get_current_attr ($) {
-    $ATTRIBUTES{ $_[0] };
-}
-
-
-=head2 install_tag TAGNAME, TAGSET
-
-Sets up TAGNAME as a tag that can be used in user templates. TAGSET is an instance of a subclass for L<Template::Declare::TagSet>.
+Sets up TAGNAME as a tag that can be used in user templates. TAGSET is an
+instance of a subclass for L<Template::Declare::TagSet>.
 
 =cut
 
@@ -519,66 +565,34 @@ sub install_tag {
     );
 }
 
+=head3 smart_tag_wrapper
 
-=head2 with
-
-C<with> is an alternative way to specify attributes for a tag:
-
-    with ( id => 'greeting', class => 'foo' ),
-        p { 'Hello, World wide web' };
-
-
-The standard way to do this is:
-
-    p { attr { id => 'greeting', class => 'foo' }
-        'Hello, World wide web' };
-
-
-=cut
-
-sub with (@) {
-    %ATTRIBUTES = ();
-    while ( my ( $key, $val ) = splice( @_, 0, 2 ) ) {
-        no warnings 'uninitialized';
-        $ATTRIBUTES{$key} = $val;
-
-        if ( lc($key) eq 'id' ) {
-            if ( $ELEMENT_ID_CACHE{$val}++ ) {
-                warn
-                    "HTML appears to contain illegal duplicate element id: $val";
-            }
-        }
+    # create a tag that has access to the arguments set with L</with>.
+    sub sample_smart_tag (&) {
+        my $code = shift;
 
+        smart_tag_wrapper {
+            my %args = @_; # set using 'with'
+            outs( 'keys: ' . join( ', ', sort keys %args) . "\n" );
+            $code->();
+        };
     }
-    wantarray ? () : '';
-}
 
-=head2 smart_tag_wrapper
-
-  # create a tag that has access to the arguments set with with.
-  sub sample_smart_tag (&) {
-      my $code = shift;
-
-      smart_tag_wrapper {
-          my %args = @_; # set using 'with'
-          outs( 'keys: ' . join( ', ', sort keys %args) . "\n" );
-          $code->();
-      };
-  }
-
-  # use it
-  with ( foo => 'bar', baz => 'bundy' ),
-    sample_smart_tag {
-      outs( "Hello, World!\n" );
+    # use it
+    with ( foo => 'bar', baz => 'bundy' ), sample_smart_tag {
+        outs( "Hello, World!\n" );
     };
 
-  # output would be
-  keys: baz, foo
-  Hello, World!
+The output would be
 
-The smart tag wrapper allows you to create code that has access to the arguments
-set using 'with', it passes them in to the wrapped code in C<@_>. It also takes
-care of putting the output in the right place and tidying up after itself.
+    keys: baz, foo
+    Hello, World!
+
+The smart tag wrapper allows you to create code that has access to the
+attribute arguments specified via C<with>. It passes those arguments in to the
+wrapped code in C<@_>. It also takes care of putting the output in the right
+place and tidying up after itself. This might be useful to change the behavior
+of a template based on attributes passed to C<with>.
 
 =cut
 
@@ -591,27 +605,169 @@ sub smart_tag_wrapper (&) {
     my %attr = %ATTRIBUTES;
     %ATTRIBUTES = ();                              # prevent leakage
 
-    my $last = join '',    #
-        map { ref($_) ? $_ : _postprocess($_) }    #
+    my $last = join '',
+        map { ref($_) ? $_ : _postprocess($_) }
         $coderef->(%attr);
 
     my $content = Template::Declare->buffer->pop;
     $content .= "$last" if not length $content and length $last;
-    Template::Declare->buffer->append( $content ) ;
+    Template::Declare->buffer->append( $content );
 
     return '';
 }
 
+=head3 create_wrapper WRAPPERNAME => sub { 'Implementation' };
+
+    create_wrapper basics => sub {
+        my $code = shift;
+        html {
+            head { title { 'Welcome' } };
+            body { $code->() }
+        }
+    };
+
+C<create_wrapper> declares a wrapper subroutine that can be called like a tag
+sub, but can optionally take arguments to be passed to the wrapper sub. For
+example, if you wanted to wrap all of the output of a template in the usual
+HTML headers and footers, you can do something like this:
+
+    package MyApp::Templates;
+    use Template::Declare::Tags;
+    use base 'Template::Declare';
+
+    BEGIN {
+        create_wrapper wrap => sub {
+            my $code = shift;
+            my %params = @_;
+            html {
+                head { title { outs "Hello, $params{user}!"} };
+                body {
+                    $code->();
+                    div { outs 'This is the end, my friend' };
+                };
+            }
+        };
+    }
+
+    template inner => sub {
+        wrap {
+            h1 { outs "Hello, Jesse, s'up?" };
+        } user => 'Jesse';
+    };
+
+Note how the C<wrap> wrapper function is available for calling after it has
+been declared in a C<BEGIN> block. Also note how you can pass arguments to the
+function after the closing brace (you don't need a comma there!).
+
+The output from the "inner" template will look something like this:
+
+ <html>
+  <head>
+   <title>Hello, Jesse!</title>
+  </head>
+  <body>
+   <h1>Hello, Jesse, s&#39;up?</h1>
+   <div>This is the end, my friend</div>
+  </body>
+ </html>
+
+=cut
+
+sub create_wrapper ($$) {
+    my $wrapper_name   = shift;
+    my $coderef        = shift;
+    my $template_class = caller;
+
+    # Shove the code ref into the calling class.
+    no strict 'refs';
+    *{"$template_class\::$wrapper_name"} = sub (&;@) { goto $coderef };
+}
+
+=head2 Helpers
+
+=head3 xml_decl HASH
+
+    xml_decl { 'xml', version => '1.0' };
+
+Emits an XML declaration. For example:
+
+    xml_decl { 'xml', version => '1.0' };
+    xml_decl { 'xml-stylesheet',  href => "chrome://global/skin/", type => "text/css" };
+
+Produces:
+
+ <?xml version="1.0"?>
+ <?xml-stylesheet href="chrome://global/skin/" type="text/css"?>
+
+=cut
+
+sub xml_decl (&;$) {
+    my $code = shift;
+    my @rv   = $code->();
+    my $name = shift @rv;
+    outs_raw("<?$name");
+    while ( my ( $field, $val ) = splice( @rv, 0, 2 ) ) {
+        outs_raw(qq/ $field="$val"/);
+    }
+    outs_raw("?>\n");
+    return @_;
+}
+
+=head3 current_template
+
+    my $path = current_template();
+
+Returns the absolute path of the current template
+
+=cut
+
+sub current_template {
+    return $TEMPLATE_STACK[-1] || '';
+}
+
+=head3 under
+
+C<under> is a helper function providing semantic sugar for the C<mix> method
+of L<Template::Declare|Template::Declare/"mix">.
+
+=cut
+
+sub under ($) { return shift }
+
+=head3 setting
+
+C<setting> is a helper function providing semantic sugar for the C<mix> method
+of L<Template::Declare|Template::Declare/"mix">.
+
+=cut
+
+sub setting ($) { return shift }
+
+=begin comment
+
+=head2 get_current_attr
+
+Deprecated.
+
+=end comment
+
+=cut
+
+sub get_current_attr ($) {
+    $ATTRIBUTES{ $_[0] };
+}
+
 sub _tag {
     my $tagset    = shift;
-    my $tag = shift;
+    my $tag       = shift;
     my $code      = shift;
     my $more_code = shift;
     $tag = $tagset->namespace . ":$tag" if defined $tagset->namespace;
 
     Template::Declare->buffer->append(
               "\n" 
-            . ( " " x $TAG_NEST_DEPTH ) . "<$tag"
+            . ( " " x $TAG_NEST_DEPTH )
+            . "<$tag"
             . join( '',
             map { qq{ $_="} . ( $ATTRIBUTES{$_} || '' ) . qq{"} }
                 sort keys %ATTRIBUTES )
@@ -632,7 +788,7 @@ sub _tag {
             $field =~ s/_/-/g;    # http_equiv is 'bar' ====> http-equiv="bar"
 
             # Squash empty values, but not '0' values
-            my $val = join( ' ', grep { defined $_ && $_ ne '' } @_ );
+            my $val = join ' ', grep { defined $_ && $_ ne '' } @_;
 
             append_attr( $field, $val );
         };
@@ -652,7 +808,7 @@ sub _tag {
     }
     my $content = Template::Declare->buffer->pop;
     $content .= "$last" if not length $content and length $last;
-    Template::Declare->buffer->append($attrs);
+    Template::Declare->buffer->append($attrs) if length $attrs;
 
     if (length $content) {
         Template::Declare->buffer->append(">$content");
@@ -670,58 +826,6 @@ sub _tag {
         : '';
 }
 
-=head2 show [$template_name or $template_coderef], args
-
-C<show> displays templates. C<args> will be passed directly to the
-template.
-
-C<show> can either be called with a template name or a package/object
-and a template.  (It's both functional and OO.)
-
-If called from within a Template::Declare subclass, then private
-templates are accessible and visible. If called from something that
-isn't a Template::Declare, only public templates wil be visible.
-
-From the outside world, users can either call
-C<Template::Declare->show()> or C<Template::Declare::tags::show()> to
-render a publicly visible template.
-
-"private" templates may only be called from within the
-C<Template::Declare> package.
-
-=cut
-
-sub show {
-    my $template = shift;
-
-    # if we're inside a template, we should show private templates
-    if ( caller->isa('Template::Declare') ) {
-        _show_template( $template, 1, \@_ );
-        return Template::Declare->buffer->data;
-    } else {
-        show_page( $template, @_);
-    }
-
-}
-
-sub show_page {
-    my $template = shift;
-    my $args = \@_;
-
-    if (defined wantarray) {
-        Template::Declare->buffer->push( private => 1, from => "T::D path $template" );
-        _show_template( $template, 0, $args );
-        %ELEMENT_ID_CACHE = ();
-        return Template::Declare->buffer->pop;
-    } else {
-        Template::Declare->buffer->push( from => "T::D path $template" );
-        _show_template( $template, 0, $args );
-        %ELEMENT_ID_CACHE = ();
-        Template::Declare->buffer->pop;
-        return undef;
-    }
-}
-
 sub _resolve_template_path {
     my $template = shift;
 
@@ -746,17 +850,6 @@ sub _resolve_template_path {
     return join '/', @parts;
 }
 
-=head2 current_template
-
-Returns the absolute path of the current template
-
-=cut
-
-sub current_template {
-    return $TEMPLATE_STACK[-1] || '';
-}
-
-
 sub _show_template {
     my $template        = shift;
     my $inside_template = shift;
@@ -792,6 +885,25 @@ sub _show_template {
     return;
 }
 
+sub _outs {
+    my $raw     = shift;
+    my @phrases = (@_);
+
+    Template::Declare->buffer->push(
+        private => (defined wantarray and not wantarray), from => "T::D outs"
+    );
+
+    foreach my $item ( grep {defined} @phrases ) {
+        my $returned = ref($item) eq 'CODE'
+            ? $item->()
+            : $raw
+                ? $item
+                : _postprocess($item);
+        Template::Declare->buffer->append( $returned );
+    }
+    return Template::Declare->buffer->pop;
+}
+
 sub _postprocess {
     my $val = shift;
     my $skip_postprocess = shift;
@@ -816,45 +928,47 @@ sub _postprocess {
     return $val;
 }
 
-=head2 import 'Package' under 'path'
+=begin comment
 
-Import the templates from C<Package> into the subpath 'path' of the current package, clobbering any
-of your own package's templates that you'd already defined.
+=head2 append_attr
+
+C<append_attr> is a helper function providing an interface for setting
+attributes from within tags. But it's better to use C<attr> or C<is> to set
+your attributes. Nohting to see here, really. Move along.
+
+=end comment
 
 =cut
 
-=head2 under
-
-C<under> is a helper function for the "import" semantic sugar.
-
-=cut
-
-sub under ($) { return shift }
+sub append_attr {
+    die "Subroutine attr failed: $_[0] => '$_[1]'\n\t".
+        "(Perhaps you're using an unknown tag in the outer container?)";
+}
 
 =head1 VARIABLES
 
-=over
+=over 4
 
-=item C<< @Template::Declare::Tags::EXPORT >>
+=item C<@Template::Declare::Tags::EXPORT>
 
-Holds the names of the static subroutines exported by this class.
-tag subroutines generated from certain tag set, however,
-are not included here.
+Holds the names of the static subroutines exported by this class. Tag
+subroutines generated by tag sets, however, are not included here.
 
-=item C<< @Template::Declare::Tags::TAG_SUB_LIST >>
+=item C<@Template::Declare::Tags::TAG_SUB_LIST>
 
-Contains the names of the tag subroutines generated
-from certain tag set.
+Contains the names of the tag subroutines generated from a tag set.
 
-Note that this array won't get cleared automatically before
-a another C<< use Template::Decalre::Tags >> statement.
+Note that this array won't get cleared automatically before another
+C<< use Template::Decalre::Tags >> statement.
 
-C<@Template::Declare::Tags::TagSubs> is aliased to this
-variable for backward-compatibility.
+C<@Template::Declare::Tags::TagSubs> is aliased to this variable for
+backward-compatibility.
 
-=item C<< $Template::Declare::Tags::TAG_NEST_DEPTH >>
+=item C<$Template::Declare::Tags::TAG_NEST_DEPTH>
 
-Controls the indentation of the XML tags in the final outputs. For example, you can temporarily disable a tag's indentation by the following lines of code:
+Controls the indentation of the XML tags in the final outputs. For example,
+you can temporarily disable a tag's indentation by the following lines of
+code:
 
     body {
         pre {
@@ -865,18 +979,18 @@ Controls the indentation of the XML tags in the final outputs. For example, you
 
 It generates
 
-    <body>
-     <pre>
-    <script src="foo.js"></script>
-     </pre>
-    </body>
+ <body>
+  <pre>
+ <script src="foo.js"></script>
+  </pre>
+ </body>
 
-Note that now the C<script> tag has I<no> indentation and we've got what we want ;)
+Note that now the C<script> tag has I<no> indentation and we've got what we
+want. ;)
 
-=item C<< $Template::Declare::Tags::SKIP_XML_ESCAPING >>
+=item C<$Template::Declare::Tags::SKIP_XML_ESCAPING>
 
-Makes L<Template::Declare> skip the XML escaping
-postprocessing entirely.
+Disables XML escape postprocessing entirely. Use at your own risk.
 
 =back
 
@@ -885,14 +999,15 @@ postprocessing entirely.
 L<Template::Declare::TagSet::HTML>,
 L<Template::Declare::TagSet::XUL>, L<Template::Declare>.
 
-=head1 AUTHOR
+=head1 AUTHORS
+
+Jesse Vincent <jesse@bestpractical.com>
 
-Jesse Vincent <jesse@bestpractical.com>,
 Agent Zhang <agentzh@yahoo.cn>
 
 =head1 COPYRIGHT
 
-Copyright 2006-2007 Best Practical Solutions, LLC
+Copyright 2006-2009 Best Practical Solutions, LLC.
 
 =cut
 
